
/**
 * Module dependencies.
 */

var caTools = require('../lib/calendar-tools')
  , assert = require('assert');

var mainSeedEv = {
    title: 'Event Instances'
  , allDay: false
  , frequency: 'day'
  , recurrence: {
        'end-by': {
            after: 4
          , type: 'after'
        }
      , every: 4
      , exceptions: []
    }
}

var start = new Date(2011, 9, 30)
  , end = new Date(2011, 11, 11);

function getSeedEvent (_start, _end) {
  var newEvent = JSON.parse(JSON.stringify(mainSeedEv));

  newEvent.start = new Date(_start || start);
  newEvent.end = new Date(_end || end);
  return newEvent;
}

var normalEvent = getSeedEvent();

var seedWeeklyEvent = getSeedEvent();
seedWeeklyEvent.start = new Date(2011, 10, 8, 15);
seedWeeklyEvent.end = new Date(2011, 10, 9, 16);
seedWeeklyEvent.frequency = 'week';
seedWeeklyEvent.recurrence.every = 1;
seedWeeklyEvent.recurrence['repeat-on'] = { t: 'on' };

// seed object
var Seed = caTools.seed(seedWeeklyEvent, {
        start: start
      , end: end
    });

module.exports = {

  '_getDatePrevInstance() - #0': function() {
    var referenceDate = new Date(2011, 8, 24)
      , prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 08 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - #1': function() {
    var referenceDate = new Date(2011, 10, 9)
      , prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 08 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - #2': function() {
    var referenceDate = new Date(2011, 10, 15, 14, 59, 59)
      , prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 08 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - #3': function() {
    var referenceDate = new Date(2011, 10, 16)
      , prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 15 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - #4': function() {
    var referenceDate = new Date(2011, 10, 16, 16)
      , prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 15 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #0': function() {
    var referenceDate = new Date(2011, 8, 24)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 08 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #1': function() {
    var referenceDate = new Date(2011, 10, 9)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 08 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #2': function() {
    var referenceDate = new Date(2011, 10, 15, 14, 59, 59)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 15 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #3': function() {
    var referenceDate = new Date(2011, 10, 16)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 15 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #4': function() {
    var referenceDate = new Date(2011, 10, 16, 15, 59, 59)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 15 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - #5': function() {
    var referenceDate = new Date(2011, 10, 16, 16)
      , prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 22 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
  },

  'getElapsedInstances() - #0': function() {
    var elapsed = Seed.getElapsedInstances(start);
    assert.equal(0, elapsed);
  },

  'getElapsedInstances() - #1': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 9, 15, 59, 59));
    assert.equal(0, elapsed);
  },

  'getElapsedInstances() - #2': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 9, 16));
    assert.equal(1, elapsed);
  },

  'getElapsedInstances() - #3': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 22, 15));
    assert.equal(2, elapsed);
  },

  'getElapsedInstances() - #4': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 23, 15, 59, 59, 999));
    assert.equal(2, elapsed);
  },

  'getElapsedInstances() - #5': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 23, 15, 59, 59, 1000));
    assert.equal(3, elapsed);
  },

  'getElapsedInstances() - #0 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(start, true);
    assert.equal(0, elapsed);
  },

  'getElapsedInstances() - #1 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 9, 15, 59, 59), true);
    assert.equal(1, elapsed);
  },

  'getElapsedInstances() - #2 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 9, 16), true);
    assert.equal(1, elapsed);
  },

  'getElapsedInstances() - #3 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 22, 15), true);
    assert.equal(2, elapsed);
  },

  'getElapsedInstances() - #4 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 23, 15, 59, 59, 999), true);
    assert.equal(3, elapsed);
  },

  'getElapsedInstances() - #5 - add partial': function() {
    var elapsed = Seed.getElapsedInstances(new Date(2011, 10, 23, 15, 59, 59, 1000), true);
    assert.equal(3, elapsed);
  },

  '_getCountdown - #0': function() {
    Seed.setPeriod(start, new Date(2011, 11, 12));
    var cd = Seed._getCountdown();
    // and-by after (4)
    assert.equal(4, cd);
  },

  '_getCountdown - #1': function() {
    Seed.setPeriod(start, new Date(2011, 10, 12));
    var cd = Seed._getCountdown();
    assert.equal(1, cd);
  },

  '_getCountdown - #2': function() {
    Seed.setPeriod(start, new Date(2011, 10, 21));
    var cd = Seed._getCountdown();
    assert.equal(2, cd);
  },

  'getInstances() - #0': function() {
    Seed.setPeriod(start, end);
    var instances = Seed.getInstances();
    assert.equal(instances.length, 4);
  },

  'getInstances() - #1': function() {
    Seed.setPeriod(new Date(2011, 10, 22), new Date(2011, 10, 23));
    var instances = Seed.getInstances();
    assert.equal(instances.length, 1);
  },

  'getInstances() - #2': function() {
    Seed.setPeriod(new Date(2011, 10, 23), new Date(2011, 10, 24));
    var instances = Seed.getInstances();
    assert.equal(instances.length, 1);
  },
};
